@@ -9,8 +9,6 @@
 |----------------------------------------------------------------------------*/
 import { expect } from 'chai';
 
-import { generate, simulate } from 'simulate-event';
-
 import { JSONObject } from '@lumino/coreutils';
 
 import { CommandRegistry } from '@lumino/commands';
@@ -49,6 +47,9 @@ class LogMenuBar extends MenuBar {
   }
 }
 
+const bubbles = true;
+const cancelable = true;
+
 describe('@lumino/widgets', () => {
   const DEFAULT_CMD = 'menubar.spec.ts:defaultCmd';
 
@@ -56,12 +57,17 @@ describe('@lumino/widgets', () => {
 
   let commands: CommandRegistry;
 
-  function createMenuBar(): MenuBar {
-    let bar = new MenuBar();
+  function createMenu() {
+    let menu = new Menu({ commands });
+    // Several tests rely on this function returning a non-empty menu
+    menu.addItem({ command: DEFAULT_CMD });
+    return menu;
+  }
+
+  function createMenuBar(options?: MenuBar.IOptions): MenuBar {
+    let bar = new MenuBar(options);
     for (let i = 0; i < 3; i++) {
-      let menu = new Menu({ commands });
-      let item = menu.addItem({ command: DEFAULT_CMD });
-      menu.addItem(item);
+      let menu = createMenu();
       menu.title.label = `Menu${i}`;
       menu.title.mnemonic = 4;
       bar.addMenu(menu);
@@ -73,14 +79,30 @@ describe('@lumino/widgets', () => {
     return bar;
   }
 
+  /**
+   * Create a MenuBar that has no active menu item.
+   */
+  function createUnfocusedMenuBar(): MenuBar {
+    const bar = createMenuBar();
+    bar.activeIndex = -1;
+    return bar;
+  }
+
   before(() => {
     commands = new CommandRegistry();
+    const iconRenderer = {
+      render: (host: HTMLElement, options?: any) => {
+        const renderNode = document.createElement('div');
+        renderNode.className = 'foo';
+        host.appendChild(renderNode);
+      }
+    };
     let cmd = commands.addCommand(DEFAULT_CMD, {
       execute: (args: JSONObject) => {
         return args;
       },
       label: 'LABEL',
-      icon: 'foo',
+      icon: iconRenderer,
       className: 'bar',
       isToggled: (args: JSONObject) => {
         return true;
@@ -105,17 +127,30 @@ describe('@lumino/widgets', () => {
       it('should take no arguments', () => {
         let bar = new MenuBar();
         expect(bar).to.be.an.instanceof(MenuBar);
+        bar.dispose();
       });
 
       it('should take options for initializing the menu bar', () => {
         let renderer = new MenuBar.Renderer();
         let bar = new MenuBar({ renderer });
         expect(bar).to.be.an.instanceof(MenuBar);
+        bar.dispose();
+      });
+
+      it('should accept only isVisible option', () => {
+        const bar = new MenuBar({
+          overflowMenuOptions: { isVisible: false }
+        });
+
+        expect(bar).to.be.an.instanceOf(MenuBar);
+
+        bar.dispose();
       });
 
       it('should add the `lm-MenuBar` class', () => {
         let bar = new MenuBar();
         expect(bar.hasClass('lm-MenuBar')).to.equal(true);
+        bar.dispose();
       });
     });
 
@@ -135,13 +170,14 @@ describe('@lumino/widgets', () => {
         let renderer = Object.create(MenuBar.defaultRenderer);
         let bar = new MenuBar({ renderer });
         expect(bar.renderer).to.equal(renderer);
+        bar.dispose();
       });
     });
 
     describe('#childMenu', () => {
       it('should get the child menu of the menu bar', () => {
         let bar = new MenuBar();
-        let menu = new Menu({ commands });
+        let menu = createMenu();
         bar.addMenu(menu);
         bar.activeIndex = 0;
         bar.openActiveMenu();
@@ -164,13 +200,14 @@ describe('@lumino/widgets', () => {
         let bar = new MenuBar();
         let content = bar.contentNode;
         expect(content.classList.contains('lm-MenuBar-content')).to.equal(true);
+        bar.dispose();
       });
     });
 
     describe('#activeMenu', () => {
       it('should get the active menu of the menu bar', () => {
         let bar = new MenuBar();
-        let menu = new Menu({ commands });
+        let menu = createMenu();
         bar.addMenu(menu);
         bar.activeIndex = 0;
         expect(bar.activeMenu).to.equal(menu);
@@ -187,7 +224,7 @@ describe('@lumino/widgets', () => {
 
       it('should set the currently active menu', () => {
         let bar = new MenuBar();
-        let menu = new Menu({ commands });
+        let menu = createMenu();
         bar.addMenu(menu);
         bar.activeMenu = menu;
         expect(bar.activeMenu).to.equal(menu);
@@ -206,7 +243,7 @@ describe('@lumino/widgets', () => {
     describe('#activeIndex', () => {
       it('should get the index of the currently active menu', () => {
         let bar = new MenuBar();
-        let menu = new Menu({ commands });
+        let menu = createMenu();
         bar.addMenu(menu);
         bar.activeMenu = menu;
         expect(bar.activeIndex).to.equal(0);
@@ -223,7 +260,7 @@ describe('@lumino/widgets', () => {
 
       it('should set the index of the currently active menu', () => {
         let bar = new MenuBar();
-        let menu = new Menu({ commands });
+        let menu = createMenu();
         bar.addMenu(menu);
         bar.activeIndex = 0;
         expect(bar.activeIndex).to.equal(0);
@@ -248,6 +285,15 @@ describe('@lumino/widgets', () => {
         expect(bar.activeIndex).to.equal(0);
         bar.dispose();
       });
+
+      it('should set to `-1` if menu at index is empty', () => {
+        let bar = createMenuBar();
+        let emptyMenu = new Menu({ commands });
+        bar.insertMenu(1, emptyMenu);
+        bar.activeIndex = 1;
+        expect(bar.activeIndex).to.equal(-1);
+        bar.dispose();
+      });
     });
 
     describe('#menus', () => {
@@ -261,6 +307,7 @@ describe('@lumino/widgets', () => {
         expect(menus.length).to.equal(2);
         expect(menus[0]).to.equal(menu0);
         expect(menus[1]).to.equal(menu1);
+        bar.dispose();
       });
     });
 
@@ -432,12 +479,14 @@ describe('@lumino/widgets', () => {
         bar.addMenu(new Menu({ commands }));
         bar.clearMenus();
         expect(bar.menus).to.eql([]);
+        bar.dispose();
       });
 
       it('should be a no-op if there are no menus', () => {
         let bar = new MenuBar();
         bar.clearMenus();
         expect(bar.menus).to.eql([]);
+        bar.dispose();
       });
     });
 
@@ -453,51 +502,158 @@ describe('@lumino/widgets', () => {
       });
 
       context('keydown', () => {
+        it('should bail on Tab', () => {
+          let event = new KeyboardEvent('keydown', {
+            cancelable: true,
+            keyCode: 9
+          });
+          bar.node.dispatchEvent(event);
+          expect(event.defaultPrevented).to.equal(false);
+        });
+
         it('should open the active menu on Enter', () => {
           let menu = bar.activeMenu!;
-          simulate(bar.node, 'keydown', { keyCode: 13 });
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 13
+            })
+          );
+          expect(menu.isAttached).to.equal(true);
+        });
+
+        it('should open the active menu on Space', () => {
+          let menu = bar.activeMenu!;
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 32
+            })
+          );
           expect(menu.isAttached).to.equal(true);
         });
 
         it('should open the active menu on Up Arrow', () => {
           let menu = bar.activeMenu!;
-          simulate(bar.node, 'keydown', { keyCode: 38 });
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 38
+            })
+          );
           expect(menu.isAttached).to.equal(true);
         });
 
         it('should open the active menu on Down Arrow', () => {
           let menu = bar.activeMenu!;
-          simulate(bar.node, 'keydown', { keyCode: 40 });
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 40
+            })
+          );
           expect(menu.isAttached).to.equal(true);
         });
 
         it('should close the active menu on Escape', () => {
           let menu = bar.activeMenu!;
           bar.openActiveMenu();
-          simulate(bar.node, 'keydown', { keyCode: 27 });
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 27
+            })
+          );
           expect(menu.isAttached).to.equal(false);
           expect(menu.activeIndex).to.equal(-1);
           expect(menu.node.contains(document.activeElement)).to.equal(false);
         });
 
         it('should activate the previous menu on Left Arrow', () => {
-          simulate(bar.node, 'keydown', { keyCode: 37 });
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 37
+            })
+          );
           expect(bar.activeIndex!).to.equal(2);
-          simulate(bar.node, 'keydown', { keyCode: 37 });
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 37
+            })
+          );
           expect(bar.activeIndex!).to.equal(1);
         });
 
+        it('should skip past empty menu on Left Arrow', () => {
+          let emptyMenu = new Menu({ commands });
+          bar.insertMenu(0, emptyMenu);
+          // Update DOM so we can focus the item node.
+          MessageLoop.flush();
+          // Call .focus() because keyboard shortcuts work from the item that
+          // has focus which may not match `bar.activeIndex`. Focus the item
+          // just to the right of the item that matches the empty menu.
+          (bar.contentNode.children[1] as HTMLElement).focus();
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 37
+            })
+          );
+          expect(bar.activeIndex).to.equal(bar.menus.length - 1);
+        });
+
         it('should activate the next menu on Right Arrow', () => {
-          simulate(bar.node, 'keydown', { keyCode: 39 });
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 39
+            })
+          );
           expect(bar.activeIndex!).to.equal(1);
-          simulate(bar.node, 'keydown', { keyCode: 39 });
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 39
+            })
+          );
           expect(bar.activeIndex!).to.equal(2);
-          simulate(bar.node, 'keydown', { keyCode: 39 });
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 39
+            })
+          );
           expect(bar.activeIndex!).to.equal(0);
         });
 
+        it('should skip past empty menu on Right Arrow', () => {
+          let emptyMenu = new Menu({ commands });
+          bar.addMenu(emptyMenu);
+          // Update DOM so we can focus the item node.
+          MessageLoop.flush();
+          let emptyIndex = bar.menus.indexOf(emptyMenu);
+          // Call .focus() because keyboard shortcuts work from the item that
+          // has focus which may not match `bar.activeIndex`. Focus the item
+          // just to the left of the item that matches the empty menu.
+          (bar.contentNode.children[emptyIndex - 1] as HTMLElement).focus();
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 39
+            })
+          );
+          expect(bar.activeIndex).to.equal(0);
+        });
+
         it('should open the menu matching a mnemonic', () => {
-          simulate(bar.node, 'keydown', { keyCode: 97 }); // '1';
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 97 // `1` key
+            })
+          );
           expect(bar.activeIndex!).to.equal(1);
           let menu = bar.activeMenu!;
           expect(menu.isAttached).to.equal(true);
@@ -505,7 +661,12 @@ describe('@lumino/widgets', () => {
 
         it('should select the next menu matching by first letter', () => {
           bar.activeIndex = 1;
-          simulate(bar.node, 'keydown', { keyCode: 77 }); // 'M';
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 77 // `M` key
+            })
+          );
           expect(bar.activeIndex!).to.equal(1);
           let menu = bar.activeMenu!;
           expect(menu.isAttached).to.equal(false);
@@ -516,35 +677,49 @@ describe('@lumino/widgets', () => {
           menu.title.label = 'Test1';
           menu.title.mnemonic = 4;
           bar.addMenu(menu);
-          simulate(bar.node, 'keydown', { keyCode: 97 }); // '1';
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 97 // `1` key
+            })
+          );
           expect(bar.activeIndex).to.equal(1);
           menu = bar.activeMenu!;
           expect(menu.isAttached).to.equal(false);
         });
 
         it('should select the only menu matching the first letter', () => {
-          let menu = new Menu({ commands });
+          let menu = createMenu();
           menu.title.label = 'Test1';
           bar.addMenu(menu);
           bar.addMenu(new Menu({ commands }));
-          simulate(bar.node, 'keydown', { keyCode: 84 }); // 'T';
+          bar.node.dispatchEvent(
+            new KeyboardEvent('keydown', {
+              bubbles,
+              keyCode: 84 // `T` key
+            })
+          );
           expect(bar.activeIndex).to.equal(3);
-          menu = bar.activeMenu!;
+          expect(bar.activeMenu).to.equal(menu);
           expect(menu.isAttached).to.equal(false);
         });
       });
 
       context('mousedown', () => {
         it('should bail if the mouse press was not on the menu bar', () => {
-          let evt = generate('mousedown', { clientX: -10 });
-          bar.node.dispatchEvent(evt);
-          expect(evt.defaultPrevented).to.equal(false);
+          let event = new MouseEvent('mousedown', {
+            bubbles,
+            cancelable: true,
+            clientX: -10
+          });
+          bar.node.dispatchEvent(event);
+          expect(event.defaultPrevented).to.equal(false);
         });
 
         it('should close an open menu if the press was not on an item', () => {
           bar.openActiveMenu();
           let menu = bar.activeMenu!;
-          simulate(bar.node, 'mousedown');
+          bar.node.dispatchEvent(new MouseEvent('mousedown', { bubbles }));
           expect(bar.activeIndex).to.equal(-1);
           expect(menu.isAttached).to.equal(false);
         });
@@ -552,16 +727,23 @@ describe('@lumino/widgets', () => {
         it('should close an active menu', () => {
           bar.openActiveMenu();
           let menu = bar.activeMenu!;
-          let node = bar.node.getElementsByClassName(
+          let firstItemNode = bar.node.getElementsByClassName(
             'lm-MenuBar-item'
           )[0] as HTMLElement;
-          let rect = node.getBoundingClientRect();
-          simulate(bar.node, 'mousedown', {
+          let rect = firstItemNode.getBoundingClientRect();
+          let mouseEvent = new MouseEvent('mousedown', {
+            bubbles,
+            cancelable: true,
             clientX: rect.left,
             clientY: rect.top
           });
           expect(bar.activeIndex).to.equal(0);
+          bar.node.dispatchEvent(mouseEvent);
+          expect(bar.activeIndex).to.equal(0);
           expect(menu.isAttached).to.equal(false);
+          // Ensure that mousedown default is not prevented, to allow browser
+          // focus to go to an item that a user clicks in the menu bar.
+          expect(mouseEvent.defaultPrevented).to.equal(false);
         });
 
         it('should open an active menu', () => {
@@ -570,12 +752,20 @@ describe('@lumino/widgets', () => {
             'lm-MenuBar-item'
           )[0] as HTMLElement;
           let rect = node.getBoundingClientRect();
-          simulate(bar.node, 'mousedown', {
+          let mouseEvent = new MouseEvent('mousedown', {
+            bubbles,
+            cancelable: true,
             clientX: rect.left,
             clientY: rect.top
           });
+          bar.node.dispatchEvent(mouseEvent);
           expect(bar.activeIndex).to.equal(0);
           expect(menu.isAttached).to.equal(true);
+          // When opening a menu, be sure to prevent default during the
+          // mousedown so that the item being clicked in the menu bar does not
+          // "steal" focus from the menu being opened.
+          mouseEvent.preventDefault();
+          expect(mouseEvent.defaultPrevented).to.equal(true);
         });
 
         it('should not close an active menu if not a left mouse press', () => {
@@ -585,14 +775,39 @@ describe('@lumino/widgets', () => {
             'lm-MenuBar-item'
           )[0] as HTMLElement;
           let rect = node.getBoundingClientRect();
-          simulate(bar.node, 'mousedown', {
-            button: 1,
-            clientX: rect.left,
-            clientY: rect.top
-          });
+          bar.node.dispatchEvent(
+            new MouseEvent('mousedown', {
+              bubbles,
+              button: 1,
+              clientX: rect.left,
+              clientY: rect.top
+            })
+          );
           expect(bar.activeIndex).to.equal(0);
           expect(menu.isAttached).to.equal(true);
         });
+
+        it('should not work on a menu bar item whose menu is empty', () => {
+          let emptyMenu = new Menu({ commands });
+          // Add title to empty menu, otherwise it will have zero width in the
+          // menu bar, which makes it impossible to test with mousedown.
+          emptyMenu.title.label = 'Empty Menu';
+          bar.insertMenu(0, emptyMenu);
+          let node = bar.node.getElementsByClassName(
+            'lm-MenuBar-item'
+          )[0] as HTMLElement;
+          let rect = node.getBoundingClientRect();
+          bar.node.dispatchEvent(
+            new MouseEvent('mousedown', {
+              bubbles,
+              button: 0,
+              clientX: rect.left,
+              clientY: rect.top
+            })
+          );
+          expect(bar.activeIndex).to.equal(-1);
+          expect(emptyMenu.isAttached).to.equal(false);
+        });
       });
 
       context('mousemove', () => {
@@ -603,10 +818,13 @@ describe('@lumino/widgets', () => {
             'lm-MenuBar-item'
           )[1] as HTMLElement;
           let rect = node.getBoundingClientRect();
-          simulate(node, 'mousemove', {
-            clientX: rect.left + 1,
-            clientY: rect.top
-          });
+          bar.node.dispatchEvent(
+            new MouseEvent('mousemove', {
+              bubbles,
+              clientX: rect.left + 1,
+              clientY: rect.top
+            })
+          );
           expect(bar.activeIndex).to.equal(1);
           expect(menu.isAttached).to.equal(false);
           expect(bar.activeMenu!.isAttached).to.equal(true);
@@ -619,10 +837,13 @@ describe('@lumino/widgets', () => {
             'lm-MenuBar-item'
           )[0] as HTMLElement;
           let rect = node.getBoundingClientRect();
-          simulate(bar.node, 'mousemove', {
-            clientX: rect.left,
-            clientY: rect.top + 1
-          });
+          bar.node.dispatchEvent(
+            new MouseEvent('mousemove', {
+              bubbles,
+              clientX: rect.left,
+              clientY: rect.top + 1
+            })
+          );
           expect(bar.activeIndex).to.equal(0);
           expect(menu.isAttached).to.equal(true);
         });
@@ -630,22 +851,22 @@ describe('@lumino/widgets', () => {
         it('should be a no-op if the mouse is not over an item and there is a menu open', () => {
           bar.openActiveMenu();
           let menu = bar.activeMenu!;
-          simulate(bar.node, 'mousemove');
+          bar.node.dispatchEvent(new MouseEvent('mousemove', { bubbles }));
           expect(bar.activeIndex).to.equal(0);
           expect(menu.isAttached).to.equal(true);
         });
       });
 
-      context('mouseleave', () => {
+      context('focusout', () => {
         it('should reset the active index if there is no open menu', () => {
-          simulate(bar.node, 'mouseleave');
+          bar.node.dispatchEvent(new FocusEvent('focusout', { bubbles }));
           expect(bar.activeIndex).to.equal(-1);
         });
 
         it('should be a no-op if there is an open menu', () => {
           bar.openActiveMenu();
           let menu = bar.activeMenu!;
-          simulate(bar.node, 'mouseleave');
+          bar.node.dispatchEvent(new FocusEvent('focusous', { bubbles }));
           expect(bar.activeIndex).to.equal(0);
           expect(menu.isAttached).to.equal(true);
         });
@@ -653,11 +874,43 @@ describe('@lumino/widgets', () => {
 
       context('contextmenu', () => {
         it('should prevent default', () => {
-          let evt = generate('contextmenu');
-          let cancelled = !bar.node.dispatchEvent(evt);
+          let event = new MouseEvent('contextmenu', { bubbles, cancelable });
+          let cancelled = !bar.node.dispatchEvent(event);
           expect(cancelled).to.equal(true);
         });
       });
+
+      context('focus', () => {
+        it('should lose focus on tab key', () => {
+          let bar = createUnfocusedMenuBar();
+          bar.activate();
+          MessageLoop.flush();
+          expect(bar.contentNode.contains(document.activeElement)).to.equal(
+            true
+          );
+          let event = new KeyboardEvent('keydown', { keyCode: 9, bubbles });
+          bar.contentNode.dispatchEvent(event);
+          expect(bar.activeIndex).to.equal(-1);
+          bar.dispose();
+        });
+
+        it('should lose focus on shift-tab key', () => {
+          let bar = createUnfocusedMenuBar();
+          bar.activate();
+          MessageLoop.flush();
+          expect(bar.contentNode.contains(document.activeElement)).to.equal(
+            true
+          );
+          let event = new KeyboardEvent('keydown', {
+            keyCode: 9,
+            shiftKey: true,
+            bubbles
+          });
+          bar.contentNode.dispatchEvent(event);
+          expect(bar.activeIndex).to.equal(-1);
+          bar.dispose();
+        });
+      });
     });
 
     describe('#onBeforeAttach()', () => {
@@ -666,15 +919,15 @@ describe('@lumino/widgets', () => {
         let node = bar.node;
         Widget.attach(bar, document.body);
         expect(bar.methods.indexOf('onBeforeAttach')).to.not.equal(-1);
-        simulate(node, 'keydown');
+        node.dispatchEvent(new KeyboardEvent('keydown', { bubbles }));
         expect(bar.events.indexOf('keydown')).to.not.equal(-1);
-        simulate(node, 'mousedown');
+        node.dispatchEvent(new MouseEvent('mousedown', { bubbles }));
         expect(bar.events.indexOf('mousedown')).to.not.equal(-1);
-        simulate(node, 'mousemove');
+        node.dispatchEvent(new MouseEvent('mousemove', { bubbles }));
         expect(bar.events.indexOf('mousemove')).to.not.equal(-1);
-        simulate(node, 'mouseleave');
-        expect(bar.events.indexOf('mouseleave')).to.not.equal(-1);
-        simulate(node, 'contextmenu');
+        node.dispatchEvent(new FocusEvent('focusout', { bubbles }));
+        expect(bar.events.indexOf('focusout')).to.not.equal(-1);
+        node.dispatchEvent(new MouseEvent('contextmenu', { bubbles }));
         expect(bar.events.indexOf('contextmenu')).to.not.equal(-1);
         bar.dispose();
       });
@@ -687,15 +940,15 @@ describe('@lumino/widgets', () => {
         Widget.attach(bar, document.body);
         Widget.detach(bar);
         expect(bar.methods.indexOf('onBeforeAttach')).to.not.equal(-1);
-        simulate(node, 'keydown');
+        node.dispatchEvent(new KeyboardEvent('keydown', { bubbles }));
         expect(bar.events.indexOf('keydown')).to.equal(-1);
-        simulate(node, 'mousedown');
+        node.dispatchEvent(new MouseEvent('mousedown', { bubbles }));
         expect(bar.events.indexOf('mousedown')).to.equal(-1);
-        simulate(node, 'mousemove');
+        node.dispatchEvent(new MouseEvent('mousemove', { bubbles }));
         expect(bar.events.indexOf('mousemove')).to.equal(-1);
-        simulate(node, 'mouseleave');
-        expect(bar.events.indexOf('mouseleave')).to.equal(-1);
-        simulate(node, 'contextmenu');
+        node.dispatchEvent(new FocusEvent('focusout', { bubbles }));
+        expect(bar.events.indexOf('focusout')).to.equal(-1);
+        node.dispatchEvent(new MouseEvent('contextmenu', { bubbles }));
         expect(bar.events.indexOf('contextmenu')).to.equal(-1);
         bar.dispose();
       });
@@ -706,29 +959,34 @@ describe('@lumino/widgets', () => {
         let bar = createMenuBar();
         Widget.detach(bar);
         MessageLoop.sendMessage(bar, Widget.Msg.ActivateRequest);
-        expect(bar.node.contains(document.activeElement)).to.equal(false);
+        expect(bar.contentNode.contains(document.activeElement)).to.equal(
+          false
+        );
+        bar.dispose();
       });
 
       it('should focus the node if attached', () => {
-        let bar = createMenuBar();
+        let bar = createUnfocusedMenuBar();
         MessageLoop.sendMessage(bar, Widget.Msg.ActivateRequest);
-        expect(bar.node.contains(document.activeElement)).to.equal(true);
+        expect(
+          bar.contentNode.contains(document.activeElement) &&
+            bar.contentNode !== document.activeElement
+        ).to.equal(true);
+        bar.dispose();
       });
     });
 
     describe('#onUpdateRequest()', () => {
-      it('should be called when the title of a menu changes', done => {
+      it('should be called when the title of a menu changes', () => {
         let bar = new LogMenuBar();
         let menu = new Menu({ commands });
         bar.addMenu(menu);
         bar.activeIndex = 0;
-        expect(bar.methods.indexOf('onUpdateRequest')).to.equal(-1);
+        expect(bar.methods).to.not.include('onUpdateRequest');
         menu.title.label = 'foo';
-        expect(bar.methods.indexOf('onUpdateRequest')).to.equal(-1);
-        requestAnimationFrame(() => {
-          expect(bar.methods.indexOf('onUpdateRequest')).to.not.equal(-1);
-          done();
-        });
+        MessageLoop.flush();
+        expect(bar.methods).to.include('onUpdateRequest');
+        bar.dispose();
       });
 
       it('should render the content', () => {
@@ -739,6 +997,47 @@ describe('@lumino/widgets', () => {
         MessageLoop.sendMessage(bar, Widget.Msg.UpdateRequest);
         let child = bar.contentNode.firstChild as HTMLElement;
         expect(child.className).to.contain('lm-MenuBar-item');
+        bar.dispose();
+      });
+
+      it('should render the overflow menu', () => {
+        let bar = createMenuBar();
+        expect(bar.overflowIndex).to.equal(-1);
+        expect(bar.overflowMenu).to.equal(null);
+        bar.node.style.maxWidth = '70px';
+        MessageLoop.sendMessage(bar, Widget.Msg.UpdateRequest);
+        MessageLoop.flush();
+        expect(bar.overflowMenu).to.not.equal(null);
+        expect(bar.overflowIndex).to.not.equal(-1);
+        bar.dispose();
+      });
+
+      it('should hide the overflow menu', () => {
+        let bar = createMenuBar();
+        expect(bar.overflowIndex).to.equal(-1);
+        expect(bar.overflowMenu).to.equal(null);
+        bar.node.style.maxWidth = '70px';
+        MessageLoop.sendMessage(bar, Widget.Msg.UpdateRequest);
+        bar.node.style.maxWidth = '400px';
+        MessageLoop.sendMessage(bar, Widget.Msg.UpdateRequest);
+        MessageLoop.flush();
+        expect(bar.overflowMenu).to.equal(null);
+        expect(bar.overflowIndex).to.equal(-1);
+        bar.dispose();
+      });
+
+      it('should not use the overflow menu', () => {
+        let bar = createMenuBar({
+          overflowMenuOptions: { isVisible: false }
+        });
+        expect(bar.overflowIndex).to.equal(-1);
+        expect(bar.overflowMenu).to.equal(null);
+        bar.node.style.maxWidth = '70px';
+        MessageLoop.sendMessage(bar, Widget.Msg.UpdateRequest);
+        MessageLoop.flush();
+        expect(bar.overflowIndex).to.equal(-1);
+        expect(bar.overflowMenu).to.equal(null);
+        bar.dispose();
       });
     });
 
@@ -774,12 +1073,14 @@ describe('@lumino/widgets', () => {
       before(() => {
         let widget = new Widget();
         widget.title.label = 'foo';
-        widget.title.icon = 'bar';
+        widget.title.iconClass = 'bar';
         widget.title.className = 'baz';
         widget.title.closable = true;
         data = {
           title: widget.title,
-          active: true
+          active: true,
+          tabbable: true,
+          disabled: false
         };
       });
 
